// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "role_side_forward.h"

#include "strategy.h"

#include "bhv_basic_offensive_kick.h"
#include "bhv_basic_move.h"
#include <rcsc/action/obsolete/body_kick_multi_step.h>
#include <rcsc/formation/formation.h>
#include <rcsc/action/neck_scan_field.h>
#include <rcsc/action/neck_turn_to_low_conf_teammate.h>
#include <rcsc/action/neck_turn_to_ball_or_scan.h>
#include <rcsc/player/player_agent.h>
#include <rcsc/player/intercept_table.h>
#include <rcsc/player/debug_client.h>
#include <rcsc/action/body_dribble.h>
#include <rcsc/action/body_intercept.h>
#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>


const std::string RoleSideForward::NAME( "SideForward" );

/*-------------------------------------------------------------------*/
/*!

 */
namespace {
rcss::RegHolder role = SoccerRole::creators().autoReg( &RoleSideForward::create,
                                                       RoleSideForward::NAME );
}

/*-------------------------------------------------------------------*/
/*!

*/
void
RoleSideForward::execute( rcsc::PlayerAgent * agent )
{
    bool kickable = agent->world().self().isKickable();
    if ( agent->world().existKickableTeammate()
         && agent->world().teammatesFromBall().front()->distFromBall()
         < agent->world().ball().distFromSelf() )
    {
        kickable = false;
    }

    if ( kickable )
    {
        doKick( agent );
    }
    else
    {
        doMove( agent );
    }

}

/*-------------------------------------------------------------------*/
/*!

*/
void
RoleSideForward::doKick( rcsc::PlayerAgent * agent )
{
    const rcsc::WorldModel & wm = agent -> world();
    
    switch ( Strategy::get_ball_area( agent -> world().ball().pos() ) ) 
    {
	//case Strategy::BA_CrossBlock:
	//case Strategy::BA_Stopper:
	//case Strategy::BA_Danger:
	//case Strategy::BA_DribbleBlock:
	//case Strategy::BA_DefMidField:
	//case Strategy::BA_DribbleAttack:
	//case Strategy::BA_OffMidField:
	case Strategy::BA_Cross:
	case Strategy::BA_ShootChance:
	    rcsc::Body_KickMultiStep( rcsc::Vector2D( 38 + rand()%3,(wm.self().pos().y ) / fabs( wm.self().pos().y ) * (rand()%3 ) ),rcsc::ServerParam::i().ballSpeedMax(),true ).execute( agent );
	    agent -> setNeckAction( new rcsc::Neck_TurnToLowConfTeammate() );
	    break;
	case Strategy::BA_DribbleAttack:
	    if( wm.self().pos().x < 48.0 && wm.self().pos().y >= 0 )
	    {
		rcsc::Body_Dribble( rcsc::Vector2D(48.0,wm.self().pos().y + 1.0 ),//posion kettei
				    7.0,//dist_thr
				    rcsc::ServerParam::i().maxPower(),
				    3,//dash_count
				    false ).execute( agent );//dodge
	    }

	    else if( wm.self().pos().x >= 48.0 )
	    {
		rcsc::Body_Dribble( rcsc::Vector2D(49.0,wm.self().pos().y - 20.0 ),//posion kettei
                                    7.0,//dist_thr
                                    rcsc::ServerParam::i().maxPower(),
                                    3,//dash_count
                                    false ).execute( agent );//dodge
	    }

	    if( wm.self().pos().x < 48.0 && wm.self().pos().y < 0 )
	    {
		rcsc::Body_Dribble( rcsc::Vector2D(48.0,wm.self().pos().y - 1.0 ),//posion kettei
                                    7.0,//dist_thr
                                    rcsc::ServerParam::i().maxPower(),
                                    3,//dash_count
                                    false ).execute( agent );//dodge
	    }

	    else if( wm.self().pos().x >= 48.0 )
	    {
		rcsc::Body_Dribble( rcsc::Vector2D(48.0,wm.self().pos().y + 20.0 ),//posion kettei
                                    7.0,//dist_thr
                                    rcsc::ServerParam::i().maxPower(),
                                    3,//dash_count
                                    false ).execute( agent );//dodge
	    }
	    
//	    rcsc::Body_Dribble( rcsc::Vector2D( 44.0,wm.self().pos().y ),
//				3.0,
//				rcsc::ServerParam::i().maxPower(),
//				4,
//				false).execute( agent );
	    agent -> setNeckAction( new rcsc::Neck_TurnToLowConfTeammate() );
	    break;

	case Strategy::BA_CrossBlock:
	case Strategy::BA_Stopper:
	case Strategy::BA_Danger:
	case Strategy::BA_DribbleBlock:
	case Strategy::BA_DefMidField:
	case Strategy::BA_OffMidField:
	    
	default:
	    Bhv_BasicOffensiveKick().execute( agent );
	    break;
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
void
RoleSideForward::doMove( rcsc::PlayerAgent * agent )
{
    const rcsc::WorldModel & wm = agent->world();

///////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////











    if( wm.ball().pos().x > 34.0 && wm.ball().pos().y > 20.0 )
    {
	rcsc::Body_Dribble( rcsc::Vector2D( 48.0,10.0 ),//posion kettei
			    7.0,//dist_thr
			    rcsc::ServerParam::i().maxPower(),
			    3,//dash_count
			    false ).execute( agent );//dodge
    }

    if( wm.ball().pos().x > 34.0 && wm.ball().pos().y < -20.0 )
    {
	rcsc::Body_Dribble( rcsc::Vector2D( 48.0,-10.0 ),//posion kettei
			    7.0,//dist_thr
			    rcsc::ServerParam::i().maxPower(),
			    3,//dash_count
			    false ).execute( agent );//dodge
    }
    
    


    int ball_reach_step = 0;
    if ( ! wm.existKickableTeammate()
         && ! wm.existKickableOpponent() )
    {
        ball_reach_step
            = std::min( wm.interceptTable()->teammateReachCycle(),
                        wm.interceptTable()->opponentReachCycle() );
    }
    const rcsc::Vector2D base_pos
        = wm.ball().inertiaPoint( ball_reach_step );

//    rcsc::Vector2D home_pos
  //      = formation().getPosition( agent->config().playerNumber(),
    //                               base_pos );
    rcsc::Vector2D home_pos = Strategy::i().getPosition( agent->world().self().unum() );
    if ( rcsc::ServerParam::i().useOffside() )
    {
        home_pos.x = std::min( home_pos.x, wm.offsideLineX() - 1.0 );
    }
    
    rcsc::dlog.addText( rcsc::Logger::ROLE,
                        "%s: HOME POSITION=(%.2f, %.2f) base_point(%.1f %.1f)"
                        ,__FILE__,
                        home_pos.x, home_pos.y,
                        base_pos.x, base_pos.y );

    switch ( Strategy::get_ball_area( base_pos ) ) {
    case Strategy::BA_CrossBlock:
    case Strategy::BA_Stopper:
    case Strategy::BA_Danger:
    case Strategy::BA_DribbleBlock:
    case Strategy::BA_DefMidField:
    case Strategy::BA_DribbleAttack:
    case Strategy::BA_OffMidField:
    case Strategy::BA_Cross:
    case Strategy::BA_ShootChance:
    default:
        Bhv_BasicMove( home_pos ).execute( agent );
        break;
    }
}
/*void
RoleSideForward::doMove( rcsc::PlayerAgent * agent )
{
    rcsc::Vector2D home_pos = Strategy::i().getPosition( agent->world().self().unum() );
    if ( ! home_pos.valid() ) home_pos.assign( 0.0, 0.0 );

    switch ( Strategy::get_ball_area( agent->world() ) ) {
    case Strategy::BA_CrossBlock:
    case Strategy::BA_Stopper:
    case Strategy::BA_Danger:
    case Strategy::BA_DribbleBlock:
    case Strategy::BA_DefMidField:
    case Strategy::BA_DribbleAttack:
    case Strategy::BA_OffMidField:
    case Strategy::BA_Cross:
    case Strategy::BA_ShootChance:
    default:
        Bhv_BasicMove( home_pos ).execute( agent );
        break;
    }
}
*/
